/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: google_any.proto */

#ifndef PROTOBUF_C_google_5fany_2eproto__INCLUDED
#define PROTOBUF_C_google_5fany_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Google__Protobuf__Any Google__Protobuf__Any;


/* --- enums --- */


/* --- messages --- */

struct  Google__Protobuf__Any
{
  ProtobufCMessage base;
  char *type_url;
  ProtobufCBinaryData value;
};
#define GOOGLE__PROTOBUF__ANY__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&google__protobuf__any__descriptor) \
    , (char *)protobuf_c_empty_string, {0,NULL} }


/* Google__Protobuf__Any methods */
void   google__protobuf__any__init
                     (Google__Protobuf__Any         *message);
size_t google__protobuf__any__get_packed_size
                     (const Google__Protobuf__Any   *message);
size_t google__protobuf__any__pack
                     (const Google__Protobuf__Any   *message,
                      uint8_t             *out);
size_t google__protobuf__any__pack_to_buffer
                     (const Google__Protobuf__Any   *message,
                      ProtobufCBuffer     *buffer);
Google__Protobuf__Any *
       google__protobuf__any__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   google__protobuf__any__free_unpacked
                     (Google__Protobuf__Any *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Google__Protobuf__Any_Closure)
                 (const Google__Protobuf__Any *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor google__protobuf__any__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_google_5fany_2eproto__INCLUDED */
