/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: storage_resolve.proto */

#ifndef PROTOBUF_C_storage_5fresolve_2eproto__INCLUDED
#define PROTOBUF_C_storage_5fresolve_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Spotify__Download__Proto__StorageResolveResponse Spotify__Download__Proto__StorageResolveResponse;


/* --- enums --- */

typedef enum _Spotify__Download__Proto__StorageResolveResponse__Result {
  SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__CDN = 0,
  SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__STORAGE = 1,
  SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__RESTRICTED = 3
    PROTOBUF_C__FORCE_ENUM_TO_BE_INT_SIZE(SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT)
} Spotify__Download__Proto__StorageResolveResponse__Result;

/* --- messages --- */

struct  Spotify__Download__Proto__StorageResolveResponse
{
  ProtobufCMessage base;
  Spotify__Download__Proto__StorageResolveResponse__Result result;
  size_t n_cdnurl;
  char **cdnurl;
  ProtobufCBinaryData fileid;
};
#define SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__download__proto__storage_resolve_response__descriptor) \
    , SPOTIFY__DOWNLOAD__PROTO__STORAGE_RESOLVE_RESPONSE__RESULT__CDN, 0,NULL, {0,NULL} }


/* Spotify__Download__Proto__StorageResolveResponse methods */
void   spotify__download__proto__storage_resolve_response__init
                     (Spotify__Download__Proto__StorageResolveResponse         *message);
size_t spotify__download__proto__storage_resolve_response__get_packed_size
                     (const Spotify__Download__Proto__StorageResolveResponse   *message);
size_t spotify__download__proto__storage_resolve_response__pack
                     (const Spotify__Download__Proto__StorageResolveResponse   *message,
                      uint8_t             *out);
size_t spotify__download__proto__storage_resolve_response__pack_to_buffer
                     (const Spotify__Download__Proto__StorageResolveResponse   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Download__Proto__StorageResolveResponse *
       spotify__download__proto__storage_resolve_response__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__download__proto__storage_resolve_response__free_unpacked
                     (Spotify__Download__Proto__StorageResolveResponse *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Download__Proto__StorageResolveResponse_Closure)
                 (const Spotify__Download__Proto__StorageResolveResponse *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__download__proto__storage_resolve_response__descriptor;
extern const ProtobufCEnumDescriptor    spotify__download__proto__storage_resolve_response__result__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_storage_5fresolve_2eproto__INCLUDED */
