/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: google_duration.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "google_duration.pb-c.h"
void   google__protobuf__duration__init
                     (Google__Protobuf__Duration         *message)
{
  static const Google__Protobuf__Duration init_value = GOOGLE__PROTOBUF__DURATION__INIT;
  *message = init_value;
}
size_t google__protobuf__duration__get_packed_size
                     (const Google__Protobuf__Duration *message)
{
  assert(message->base.descriptor == &google__protobuf__duration__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t google__protobuf__duration__pack
                     (const Google__Protobuf__Duration *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &google__protobuf__duration__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t google__protobuf__duration__pack_to_buffer
                     (const Google__Protobuf__Duration *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &google__protobuf__duration__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Google__Protobuf__Duration *
       google__protobuf__duration__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Google__Protobuf__Duration *)
     protobuf_c_message_unpack (&google__protobuf__duration__descriptor,
                                allocator, len, data);
}
void   google__protobuf__duration__free_unpacked
                     (Google__Protobuf__Duration *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &google__protobuf__duration__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor google__protobuf__duration__field_descriptors[2] =
{
  {
    "seconds",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(Google__Protobuf__Duration, seconds),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "nanos",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Google__Protobuf__Duration, nanos),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned google__protobuf__duration__field_indices_by_name[] = {
  1,   /* field[1] = nanos */
  0,   /* field[0] = seconds */
};
static const ProtobufCIntRange google__protobuf__duration__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor google__protobuf__duration__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "google.protobuf.Duration",
  "Duration",
  "Google__Protobuf__Duration",
  "google.protobuf",
  sizeof(Google__Protobuf__Duration),
  2,
  google__protobuf__duration__field_descriptors,
  google__protobuf__duration__field_indices_by_name,
  1,  google__protobuf__duration__number_ranges,
  (ProtobufCMessageInit) google__protobuf__duration__init,
  NULL,NULL,NULL    /* reserved[123] */
};
