/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_challenges_code.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "login5_challenges_code.pb-c.h"
void   spotify__login5__v3__challenges__code_challenge__init
                     (Spotify__Login5__V3__Challenges__CodeChallenge         *message)
{
  static const Spotify__Login5__V3__Challenges__CodeChallenge init_value = SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__challenges__code_challenge__get_packed_size
                     (const Spotify__Login5__V3__Challenges__CodeChallenge *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_challenge__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__challenges__code_challenge__pack
                     (const Spotify__Login5__V3__Challenges__CodeChallenge *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_challenge__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__challenges__code_challenge__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__CodeChallenge *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_challenge__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Challenges__CodeChallenge *
       spotify__login5__v3__challenges__code_challenge__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Challenges__CodeChallenge *)
     protobuf_c_message_unpack (&spotify__login5__v3__challenges__code_challenge__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__challenges__code_challenge__free_unpacked
                     (Spotify__Login5__V3__Challenges__CodeChallenge *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_challenge__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__login5__v3__challenges__code_solution__init
                     (Spotify__Login5__V3__Challenges__CodeSolution         *message)
{
  static const Spotify__Login5__V3__Challenges__CodeSolution init_value = SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_SOLUTION__INIT;
  *message = init_value;
}
size_t spotify__login5__v3__challenges__code_solution__get_packed_size
                     (const Spotify__Login5__V3__Challenges__CodeSolution *message)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_solution__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__login5__v3__challenges__code_solution__pack
                     (const Spotify__Login5__V3__Challenges__CodeSolution *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_solution__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__login5__v3__challenges__code_solution__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__CodeSolution *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_solution__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Login5__V3__Challenges__CodeSolution *
       spotify__login5__v3__challenges__code_solution__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Login5__V3__Challenges__CodeSolution *)
     protobuf_c_message_unpack (&spotify__login5__v3__challenges__code_solution__descriptor,
                                allocator, len, data);
}
void   spotify__login5__v3__challenges__code_solution__free_unpacked
                     (Spotify__Login5__V3__Challenges__CodeSolution *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__login5__v3__challenges__code_solution__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCEnumValue spotify__login5__v3__challenges__code_challenge__method__enum_values_by_number[2] =
{
  { "UNKNOWN", "SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD__UNKNOWN", 0 },
  { "SMS", "SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD__SMS", 1 },
};
static const ProtobufCIntRange spotify__login5__v3__challenges__code_challenge__method__value_ranges[] = {
{0, 0},{0, 2}
};
static const ProtobufCEnumValueIndex spotify__login5__v3__challenges__code_challenge__method__enum_values_by_name[2] =
{
  { "SMS", 1 },
  { "UNKNOWN", 0 },
};
const ProtobufCEnumDescriptor spotify__login5__v3__challenges__code_challenge__method__descriptor =
{
  PROTOBUF_C__ENUM_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.challenges.CodeChallenge.Method",
  "Method",
  "Spotify__Login5__V3__Challenges__CodeChallenge__Method",
  "spotify.login5.v3.challenges",
  2,
  spotify__login5__v3__challenges__code_challenge__method__enum_values_by_number,
  2,
  spotify__login5__v3__challenges__code_challenge__method__enum_values_by_name,
  1,
  spotify__login5__v3__challenges__code_challenge__method__value_ranges,
  NULL,NULL,NULL,NULL   /* reserved[1234] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__challenges__code_challenge__field_descriptors[4] =
{
  {
    "method",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_ENUM,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Challenges__CodeChallenge, method),
    &spotify__login5__v3__challenges__code_challenge__method__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "code_length",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Challenges__CodeChallenge, code_length),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "expires_in",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Challenges__CodeChallenge, expires_in),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "canonical_phone_number",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Challenges__CodeChallenge, canonical_phone_number),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__challenges__code_challenge__field_indices_by_name[] = {
  3,   /* field[3] = canonical_phone_number */
  1,   /* field[1] = code_length */
  2,   /* field[2] = expires_in */
  0,   /* field[0] = method */
};
static const ProtobufCIntRange spotify__login5__v3__challenges__code_challenge__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 4 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__challenges__code_challenge__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.challenges.CodeChallenge",
  "CodeChallenge",
  "Spotify__Login5__V3__Challenges__CodeChallenge",
  "spotify.login5.v3.challenges",
  sizeof(Spotify__Login5__V3__Challenges__CodeChallenge),
  4,
  spotify__login5__v3__challenges__code_challenge__field_descriptors,
  spotify__login5__v3__challenges__code_challenge__field_indices_by_name,
  1,  spotify__login5__v3__challenges__code_challenge__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__challenges__code_challenge__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__login5__v3__challenges__code_solution__field_descriptors[1] =
{
  {
    "code",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Login5__V3__Challenges__CodeSolution, code),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__login5__v3__challenges__code_solution__field_indices_by_name[] = {
  0,   /* field[0] = code */
};
static const ProtobufCIntRange spotify__login5__v3__challenges__code_solution__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor spotify__login5__v3__challenges__code_solution__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.login5.v3.challenges.CodeSolution",
  "CodeSolution",
  "Spotify__Login5__V3__Challenges__CodeSolution",
  "spotify.login5.v3.challenges",
  sizeof(Spotify__Login5__V3__Challenges__CodeSolution),
  1,
  spotify__login5__v3__challenges__code_solution__field_descriptors,
  spotify__login5__v3__challenges__code_solution__field_indices_by_name,
  1,  spotify__login5__v3__challenges__code_solution__number_ranges,
  (ProtobufCMessageInit) spotify__login5__v3__challenges__code_solution__init,
  NULL,NULL,NULL    /* reserved[123] */
};
