/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: entity_extension_data.proto */

/* Do not generate deprecated warnings for self */
#ifndef PROTOBUF_C__NO_DEPRECATED
#define PROTOBUF_C__NO_DEPRECATED
#endif

#include "entity_extension_data.pb-c.h"
void   spotify__extendedmetadata__entity_extension_data_header__init
                     (Spotify__Extendedmetadata__EntityExtensionDataHeader         *message)
{
  static const Spotify__Extendedmetadata__EntityExtensionDataHeader init_value = SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA_HEADER__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__entity_extension_data_header__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_header__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__entity_extension_data_header__pack
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_header__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__entity_extension_data_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionDataHeader *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_header__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__EntityExtensionDataHeader *
       spotify__extendedmetadata__entity_extension_data_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__EntityExtensionDataHeader *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__entity_extension_data_header__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__entity_extension_data_header__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionDataHeader *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data_header__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__entity_extension_data__init
                     (Spotify__Extendedmetadata__EntityExtensionData         *message)
{
  static const Spotify__Extendedmetadata__EntityExtensionData init_value = SPOTIFY__EXTENDEDMETADATA__ENTITY_EXTENSION_DATA__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__entity_extension_data__get_packed_size
                     (const Spotify__Extendedmetadata__EntityExtensionData *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__entity_extension_data__pack
                     (const Spotify__Extendedmetadata__EntityExtensionData *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__entity_extension_data__pack_to_buffer
                     (const Spotify__Extendedmetadata__EntityExtensionData *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__EntityExtensionData *
       spotify__extendedmetadata__entity_extension_data__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__EntityExtensionData *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__entity_extension_data__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__entity_extension_data__free_unpacked
                     (Spotify__Extendedmetadata__EntityExtensionData *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__entity_extension_data__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__plain_list_assoc__init
                     (Spotify__Extendedmetadata__PlainListAssoc         *message)
{
  static const Spotify__Extendedmetadata__PlainListAssoc init_value = SPOTIFY__EXTENDEDMETADATA__PLAIN_LIST_ASSOC__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__plain_list_assoc__get_packed_size
                     (const Spotify__Extendedmetadata__PlainListAssoc *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__plain_list_assoc__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__plain_list_assoc__pack
                     (const Spotify__Extendedmetadata__PlainListAssoc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__plain_list_assoc__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__plain_list_assoc__pack_to_buffer
                     (const Spotify__Extendedmetadata__PlainListAssoc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__plain_list_assoc__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__PlainListAssoc *
       spotify__extendedmetadata__plain_list_assoc__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__PlainListAssoc *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__plain_list_assoc__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__plain_list_assoc__free_unpacked
                     (Spotify__Extendedmetadata__PlainListAssoc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__plain_list_assoc__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__assoc_header__init
                     (Spotify__Extendedmetadata__AssocHeader         *message)
{
  static const Spotify__Extendedmetadata__AssocHeader init_value = SPOTIFY__EXTENDEDMETADATA__ASSOC_HEADER__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__assoc_header__get_packed_size
                     (const Spotify__Extendedmetadata__AssocHeader *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc_header__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__assoc_header__pack
                     (const Spotify__Extendedmetadata__AssocHeader *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc_header__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__assoc_header__pack_to_buffer
                     (const Spotify__Extendedmetadata__AssocHeader *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc_header__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__AssocHeader *
       spotify__extendedmetadata__assoc_header__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__AssocHeader *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__assoc_header__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__assoc_header__free_unpacked
                     (Spotify__Extendedmetadata__AssocHeader *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc_header__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
void   spotify__extendedmetadata__assoc__init
                     (Spotify__Extendedmetadata__Assoc         *message)
{
  static const Spotify__Extendedmetadata__Assoc init_value = SPOTIFY__EXTENDEDMETADATA__ASSOC__INIT;
  *message = init_value;
}
size_t spotify__extendedmetadata__assoc__get_packed_size
                     (const Spotify__Extendedmetadata__Assoc *message)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc__descriptor);
  return protobuf_c_message_get_packed_size ((const ProtobufCMessage*)(message));
}
size_t spotify__extendedmetadata__assoc__pack
                     (const Spotify__Extendedmetadata__Assoc *message,
                      uint8_t       *out)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc__descriptor);
  return protobuf_c_message_pack ((const ProtobufCMessage*)message, out);
}
size_t spotify__extendedmetadata__assoc__pack_to_buffer
                     (const Spotify__Extendedmetadata__Assoc *message,
                      ProtobufCBuffer *buffer)
{
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc__descriptor);
  return protobuf_c_message_pack_to_buffer ((const ProtobufCMessage*)message, buffer);
}
Spotify__Extendedmetadata__Assoc *
       spotify__extendedmetadata__assoc__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data)
{
  return (Spotify__Extendedmetadata__Assoc *)
     protobuf_c_message_unpack (&spotify__extendedmetadata__assoc__descriptor,
                                allocator, len, data);
}
void   spotify__extendedmetadata__assoc__free_unpacked
                     (Spotify__Extendedmetadata__Assoc *message,
                      ProtobufCAllocator *allocator)
{
  if(!message)
    return;
  assert(message->base.descriptor == &spotify__extendedmetadata__assoc__descriptor);
  protobuf_c_message_free_unpacked ((ProtobufCMessage*)message, allocator);
}
static const ProtobufCFieldDescriptor spotify__extendedmetadata__entity_extension_data_header__field_descriptors[5] =
{
  {
    "status_code",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT32,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataHeader, status_code),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "etag",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataHeader, etag),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "locale",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataHeader, locale),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "cache_ttl_in_seconds",
    4,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataHeader, cache_ttl_in_seconds),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "offline_ttl_in_seconds",
    5,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_INT64,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionDataHeader, offline_ttl_in_seconds),
    NULL,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__entity_extension_data_header__field_indices_by_name[] = {
  3,   /* field[3] = cache_ttl_in_seconds */
  1,   /* field[1] = etag */
  2,   /* field[2] = locale */
  4,   /* field[4] = offline_ttl_in_seconds */
  0,   /* field[0] = status_code */
};
static const ProtobufCIntRange spotify__extendedmetadata__entity_extension_data_header__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 5 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data_header__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.EntityExtensionDataHeader",
  "EntityExtensionDataHeader",
  "Spotify__Extendedmetadata__EntityExtensionDataHeader",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__EntityExtensionDataHeader),
  5,
  spotify__extendedmetadata__entity_extension_data_header__field_descriptors,
  spotify__extendedmetadata__entity_extension_data_header__field_indices_by_name,
  1,  spotify__extendedmetadata__entity_extension_data_header__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__entity_extension_data_header__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__entity_extension_data__field_descriptors[3] =
{
  {
    "header",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionData, header),
    &spotify__extendedmetadata__entity_extension_data_header__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "entity_uri",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_STRING,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionData, entity_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "extension_data",
    3,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__EntityExtensionData, extension_data),
    &google__protobuf__any__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__entity_extension_data__field_indices_by_name[] = {
  1,   /* field[1] = entity_uri */
  2,   /* field[2] = extension_data */
  0,   /* field[0] = header */
};
static const ProtobufCIntRange spotify__extendedmetadata__entity_extension_data__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 3 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__entity_extension_data__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.EntityExtensionData",
  "EntityExtensionData",
  "Spotify__Extendedmetadata__EntityExtensionData",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__EntityExtensionData),
  3,
  spotify__extendedmetadata__entity_extension_data__field_descriptors,
  spotify__extendedmetadata__entity_extension_data__field_indices_by_name,
  1,  spotify__extendedmetadata__entity_extension_data__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__entity_extension_data__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__plain_list_assoc__field_descriptors[1] =
{
  {
    "entity_uri",
    1,
    PROTOBUF_C_LABEL_REPEATED,
    PROTOBUF_C_TYPE_STRING,
    offsetof(Spotify__Extendedmetadata__PlainListAssoc, n_entity_uri),
    offsetof(Spotify__Extendedmetadata__PlainListAssoc, entity_uri),
    NULL,
    &protobuf_c_empty_string,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__plain_list_assoc__field_indices_by_name[] = {
  0,   /* field[0] = entity_uri */
};
static const ProtobufCIntRange spotify__extendedmetadata__plain_list_assoc__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 1 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__plain_list_assoc__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.PlainListAssoc",
  "PlainListAssoc",
  "Spotify__Extendedmetadata__PlainListAssoc",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__PlainListAssoc),
  1,
  spotify__extendedmetadata__plain_list_assoc__field_descriptors,
  spotify__extendedmetadata__plain_list_assoc__field_indices_by_name,
  1,  spotify__extendedmetadata__plain_list_assoc__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__plain_list_assoc__init,
  NULL,NULL,NULL    /* reserved[123] */
};
#define spotify__extendedmetadata__assoc_header__field_descriptors NULL
#define spotify__extendedmetadata__assoc_header__field_indices_by_name NULL
#define spotify__extendedmetadata__assoc_header__number_ranges NULL
const ProtobufCMessageDescriptor spotify__extendedmetadata__assoc_header__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.AssocHeader",
  "AssocHeader",
  "Spotify__Extendedmetadata__AssocHeader",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__AssocHeader),
  0,
  spotify__extendedmetadata__assoc_header__field_descriptors,
  spotify__extendedmetadata__assoc_header__field_indices_by_name,
  0,  spotify__extendedmetadata__assoc_header__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__assoc_header__init,
  NULL,NULL,NULL    /* reserved[123] */
};
static const ProtobufCFieldDescriptor spotify__extendedmetadata__assoc__field_descriptors[2] =
{
  {
    "header",
    1,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__Assoc, header),
    &spotify__extendedmetadata__assoc_header__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
  {
    "plain_list",
    2,
    PROTOBUF_C_LABEL_NONE,
    PROTOBUF_C_TYPE_MESSAGE,
    0,   /* quantifier_offset */
    offsetof(Spotify__Extendedmetadata__Assoc, plain_list),
    &spotify__extendedmetadata__plain_list_assoc__descriptor,
    NULL,
    0,             /* flags */
    0,NULL,NULL    /* reserved1,reserved2, etc */
  },
};
static const unsigned spotify__extendedmetadata__assoc__field_indices_by_name[] = {
  0,   /* field[0] = header */
  1,   /* field[1] = plain_list */
};
static const ProtobufCIntRange spotify__extendedmetadata__assoc__number_ranges[1 + 1] =
{
  { 1, 0 },
  { 0, 2 }
};
const ProtobufCMessageDescriptor spotify__extendedmetadata__assoc__descriptor =
{
  PROTOBUF_C__MESSAGE_DESCRIPTOR_MAGIC,
  "spotify.extendedmetadata.Assoc",
  "Assoc",
  "Spotify__Extendedmetadata__Assoc",
  "spotify.extendedmetadata",
  sizeof(Spotify__Extendedmetadata__Assoc),
  2,
  spotify__extendedmetadata__assoc__field_descriptors,
  spotify__extendedmetadata__assoc__field_indices_by_name,
  1,  spotify__extendedmetadata__assoc__number_ranges,
  (ProtobufCMessageInit) spotify__extendedmetadata__assoc__init,
  NULL,NULL,NULL    /* reserved[123] */
};
