import {
  mdiAccountMusic,
  mdiAlbum,
  mdiBookOpenPageVariant,
  mdiBookOpenVariant,
  mdiBookshelf,
  mdiBroadcast,
  mdiCancel,
  mdiCast,
  mdiCastVariant,
  mdiCheck,
  mdiChevronDown,
  mdiChevronLeft,
  mdiChevronUp,
  mdiClose,
  mdiDotsHorizontal,
  mdiDotsVertical,
  mdiDownload,
  mdiDragHorizontal,
  mdiEyeOffOutline,
  mdiFastForward30,
  mdiFileMusicOutline,
  mdiFire,
  mdiFolder,
  mdiFolderOpen,
  mdiHistory,
  mdiLoading,
  mdiMagnify,
  mdiMenu,
  mdiMicrophone,
  mdiMusic,
  mdiMusicBoxMultiple,
  mdiOpenInNew,
  mdiPause,
  mdiPencil,
  mdiPipe,
  mdiPlay,
  mdiPlaylistMusic,
  mdiPlaylistPlay,
  mdiPlaylistPlus,
  mdiRadio,
  mdiRadioTower,
  mdiRefresh,
  mdiRepeat,
  mdiRepeatOff,
  mdiRepeatOnce,
  mdiRewind10,
  mdiRss,
  mdiScriptTextOutline,
  mdiScriptTextPlay,
  mdiServer,
  mdiShuffle,
  mdiShuffleDisabled,
  mdiSkipBackward,
  mdiSkipForward,
  mdiSpeaker,
  mdiSpotify,
  mdiStop,
  mdiTimelapse,
  mdiTrashCanOutline,
  mdiVectorLink,
  mdiVolumeHigh,
  mdiVolumeOff,
  mdiWeb
} from '@mdi/js'

export const icons = {
  mdiAccountMusic,
  mdiAlbum,
  mdiBookOpenPageVariant,
  mdiBookOpenVariant,
  mdiBookshelf,
  mdiBroadcast,
  mdiCancel,
  mdiCast,
  mdiCastVariant,
  mdiCheck,
  mdiChevronDown,
  mdiChevronLeft,
  mdiChevronUp,
  mdiClose,
  mdiDotsHorizontal,
  mdiDotsVertical,
  mdiDownload,
  mdiDragHorizontal,
  mdiEyeOffOutline,
  mdiFastForward30,
  mdiFileMusicOutline,
  mdiFire,
  mdiFolder,
  mdiFolderOpen,
  mdiHistory,
  mdiLoading,
  mdiMagnify,
  mdiMenu,
  mdiMicrophone,
  mdiMusic,
  mdiMusicBoxMultiple,
  mdiOpenInNew,
  mdiPause,
  mdiPencil,
  mdiPipe,
  mdiPlay,
  mdiPlaylistMusic,
  mdiPlaylistPlay,
  mdiPlaylistPlus,
  mdiRadio,
  mdiRadioTower,
  mdiRefresh,
  mdiRepeat,
  mdiRepeatOff,
  mdiRepeatOnce,
  mdiRewind10,
  mdiRss,
  mdiScriptTextOutline,
  mdiScriptTextPlay,
  mdiServer,
  mdiShuffle,
  mdiShuffleDisabled,
  mdiSkipBackward,
  mdiSkipForward,
  mdiSpeaker,
  mdiSpotify,
  mdiStop,
  mdiTimelapse,
  mdiTrashCanOutline,
  mdiVectorLink,
  mdiVolumeHigh,
  mdiVolumeOff,
  mdiWeb
}
