/* Generated by the protocol buffer compiler.  DO NOT EDIT! */
/* Generated from: login5_challenges_code.proto */

#ifndef PROTOBUF_C_login5_5fchallenges_5fcode_2eproto__INCLUDED
#define PROTOBUF_C_login5_5fchallenges_5fcode_2eproto__INCLUDED

#include <protobuf-c/protobuf-c.h>

PROTOBUF_C__BEGIN_DECLS

#if PROTOBUF_C_VERSION_NUMBER < 1003000
# error This file was generated by a newer version of protoc-c which is incompatible with your libprotobuf-c headers. Please update your headers.
#elif 1004001 < PROTOBUF_C_MIN_COMPILER_VERSION
# error This file was generated by an older version of protoc-c which is incompatible with your libprotobuf-c headers. Please regenerate this file with a newer version of protoc-c.
#endif


typedef struct Spotify__Login5__V3__Challenges__CodeChallenge Spotify__Login5__V3__Challenges__CodeChallenge;
typedef struct Spotify__Login5__V3__Challenges__CodeSolution Spotify__Login5__V3__Challenges__CodeSolution;


/* --- enums --- */

typedef enum _Spotify__Login5__V3__Challenges__CodeChallenge__Method {
  SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD__UNKNOWN = 0,
  SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD__SMS = 1
    PROTOBUF_C__FORCE_ENUM_TO_BE_INT_SIZE(SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD)
} Spotify__Login5__V3__Challenges__CodeChallenge__Method;

/* --- messages --- */

struct  Spotify__Login5__V3__Challenges__CodeChallenge
{
  ProtobufCMessage base;
  Spotify__Login5__V3__Challenges__CodeChallenge__Method method;
  int32_t code_length;
  int32_t expires_in;
  char *canonical_phone_number;
};
#define SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__challenges__code_challenge__descriptor) \
    , SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_CHALLENGE__METHOD__UNKNOWN, 0, 0, (char *)protobuf_c_empty_string }


struct  Spotify__Login5__V3__Challenges__CodeSolution
{
  ProtobufCMessage base;
  char *code;
};
#define SPOTIFY__LOGIN5__V3__CHALLENGES__CODE_SOLUTION__INIT \
 { PROTOBUF_C_MESSAGE_INIT (&spotify__login5__v3__challenges__code_solution__descriptor) \
    , (char *)protobuf_c_empty_string }


/* Spotify__Login5__V3__Challenges__CodeChallenge methods */
void   spotify__login5__v3__challenges__code_challenge__init
                     (Spotify__Login5__V3__Challenges__CodeChallenge         *message);
size_t spotify__login5__v3__challenges__code_challenge__get_packed_size
                     (const Spotify__Login5__V3__Challenges__CodeChallenge   *message);
size_t spotify__login5__v3__challenges__code_challenge__pack
                     (const Spotify__Login5__V3__Challenges__CodeChallenge   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__challenges__code_challenge__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__CodeChallenge   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Challenges__CodeChallenge *
       spotify__login5__v3__challenges__code_challenge__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__challenges__code_challenge__free_unpacked
                     (Spotify__Login5__V3__Challenges__CodeChallenge *message,
                      ProtobufCAllocator *allocator);
/* Spotify__Login5__V3__Challenges__CodeSolution methods */
void   spotify__login5__v3__challenges__code_solution__init
                     (Spotify__Login5__V3__Challenges__CodeSolution         *message);
size_t spotify__login5__v3__challenges__code_solution__get_packed_size
                     (const Spotify__Login5__V3__Challenges__CodeSolution   *message);
size_t spotify__login5__v3__challenges__code_solution__pack
                     (const Spotify__Login5__V3__Challenges__CodeSolution   *message,
                      uint8_t             *out);
size_t spotify__login5__v3__challenges__code_solution__pack_to_buffer
                     (const Spotify__Login5__V3__Challenges__CodeSolution   *message,
                      ProtobufCBuffer     *buffer);
Spotify__Login5__V3__Challenges__CodeSolution *
       spotify__login5__v3__challenges__code_solution__unpack
                     (ProtobufCAllocator  *allocator,
                      size_t               len,
                      const uint8_t       *data);
void   spotify__login5__v3__challenges__code_solution__free_unpacked
                     (Spotify__Login5__V3__Challenges__CodeSolution *message,
                      ProtobufCAllocator *allocator);
/* --- per-message closures --- */

typedef void (*Spotify__Login5__V3__Challenges__CodeChallenge_Closure)
                 (const Spotify__Login5__V3__Challenges__CodeChallenge *message,
                  void *closure_data);
typedef void (*Spotify__Login5__V3__Challenges__CodeSolution_Closure)
                 (const Spotify__Login5__V3__Challenges__CodeSolution *message,
                  void *closure_data);

/* --- services --- */


/* --- descriptors --- */

extern const ProtobufCMessageDescriptor spotify__login5__v3__challenges__code_challenge__descriptor;
extern const ProtobufCEnumDescriptor    spotify__login5__v3__challenges__code_challenge__method__descriptor;
extern const ProtobufCMessageDescriptor spotify__login5__v3__challenges__code_solution__descriptor;

PROTOBUF_C__END_DECLS


#endif  /* PROTOBUF_C_login5_5fchallenges_5fcode_2eproto__INCLUDED */
